// MQODG.CPP program

//*********************************************************************************************
//*                                                                                           *
//*                            MQODG.CPP program                                              *
//*                            =================                                              *
//*                                                                                           *
//*   This program will generate a file of text-mode records of a specified length            *
//*        and a specified size. The records may either be space-filled, or contain           *
//*        a selection of varying digits.                                                     *
//*                                                                                           *
//*   Each record starts with   <number-of-record,record-length>                              *
//*        and    ends  with    *                                                             *
//*                                                                                           *
//*                                                                                           *
//*   The parameters are  /Faaaaaa    -> the name of the file to generate (8.3 format)        *
//*                                                  default -> 12345678.123                  *
//*                                                                                           *
//*                       /Lnnnnnn    -> the LRECL of the file min 40                         *
//*                                                            max 65535                      *
//*                                                            def 80                         *
//*                                                                                           *
//*                       /Nnnnnnn    -> the number of records to generate min 10             *
//*                                                                        max 65535          *
//*                                                                        def 100            *
//*                                                                                           *
//*                       /Snnnnnn    -> controls the filling of the records                  *
//*                                           /S0   -> space fill them (default)              *
//*                                           else  -> randomly fill with numeric digits      *
//*                                                             based on this seed number     *
//*                                                                                           *
//*                                                                                           *
//*   Example : MQODG /Frah.f1 /L80 /N500  will create a file called rah.f1                   *
//*                                             containing 500 80 byte records with           *
//*                                             spaces in them                                *
//*                                                                                           *
//*                                                                                           *
//*   Standard messages are produced to show the parameter settings and notify errors.        *
//*                                                                                           *
//*                                                                                           *
//*   Return Codes :     0   -> OK                                                            *
//*                      4   -> Getmain for the Buffer used to build the records failed       *
//*                      8   -> File could not be opened for output                           *
//*                      12  -> Writing of a record failed                                    *
//*                      16  -> File could not be closed                                      *
//*                                                                                           *
//*                                                                                           *
//*   Copyright : This program is Copyrighted by IBM UK Labs Limited, 1994.                   *
//*                                                                                           *
//*                                                                                           *
//*   Support   : This program is not supported in any way by IBM. It is distributed          *
//*               only to show techniques for Message Queueing. It may be freely              *
//*               modified and adapted by your installation.                                  *
//*                                                                                           *
//*   Author    : Robert Harris,                                                              *
//*               Object Technology Products,                                                 *
//*               IBM Hursley Park,                                                           *
//*               England.                                                                    *
//*                                                                                           *
//*********************************************************************************************

//
// System Includes
//

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>

//
// Windows includes
//

#include <windows.h>

//
// Definitions of datatypes
//

#define FULLWORD     long
#define HALFWORD     int
#define MAXSFULLWORD (68719476735)
#define MAXUFULLWORD (4294967295)
#define MAXSHALFWORD (4095)
#define MAXUHALFWORD (65535)

//
// The parms class contains the unravelling of the
//     supplied parameters
//

class parms
{
   private:

       HALFWORD    lrecl            ;       // from /L
       HALFWORD    numrec           ;       //      /N
       HALFWORD    seed             ;       //      /S
       char        filename??(13??) ;       //      /F

       FULLWORD parms::getnumb( char * const ptr ) ;    //Private func to get nums out of parms

   public:
                 parms       () { lrecl      = 80                 ; //Constructor
                                  numrec     = 100                ; //  sets the parm
                                  seed       = 0                  ; //  defaults
                                  strcpy(filename,"12345678.123") ;
                                }

       HALFWORD  getlrecl    () { return(lrecl)  ; }                //Access func to the
       HALFWORD  getnumrec   () { return(numrec) ; }                //parms
       HALFWORD  getseed     () { return(seed)   ; }

       char    * getfilename (char * const dest)
                             {
                               strcpy(dest,filename) ;
                               return(dest)          ;
                             }


       void     parmset ( int     const argc,                       //Function to set
                          char ** const argv,                       // up parms in the
                          char ** const envp  ) ;                   // class

} ;

//
// The parmset function is the part of the parms class
//     that builds the parameters
//
// Valid parms are :
//
//            /Lnnnnn   -> LRECL for the file
//            /Nnnnnn   -> Number of records in the file
//            /Snnnnn   -> Controls random filling of the file (0=space fill)
//            /Faaaaa   -> Name of the file to generate (8.3 format)
//
void parms::parmset ( int     const argc,
                      char ** const argv,
                      char ** const envp  )
 {
    int     i = 0   ;
    char  * p = 0   ;
    char    c = ' ' ;
    char    s = ' ' ;
    char    v = ' ' ;

    for ( i=1 ; i < argc ; i++ )     // loop through all the parms
        {
          p = argv??(i??) ;          // start of current parm
          c = *p          ;          // should be the selector, so ignore it
          s = *(++p)      ;          // the parm key
          p++             ;          // start of parm data
          v = *p          ;          // first of the parm data

          switch ( s )
           {
             case    'f'  :      // File name
             case    'F'  :

                             if ( strlen(p) <= 12 )
                                  strcpy(this->filename,p)    ;
                                  else
                                  cout << "/F parm too long. "
                                       << this->filename
                                       << " used\n"           ;
                             break                            ;

             case    'l'  :      // LRECL
             case    'L'  :

                             if ( getnumb(p) <= MAXUHALFWORD )
                                  this->lrecl  = getnumb(p)              ;
                                  else
                                  cout << "/L parm too big. "
                                       << this->lrecl
                                       << " used\n"                      ;
                             if ( this->lrecl <  40 )
                                {
                                  this->lrecl  = 40                      ;
                                  cout << "/L parm too small. 40 used\n" ;
                                }
                             break                                       ;

             case    'n'  :      // Number of records
             case    'N'  :

                             if ( getnumb(p) <= MAXUHALFWORD )
                                  this->numrec = getnumb(p)              ;
                                  else
                                  cout << "/N parm too big. "
                                       << this->numrec
                                       << " used\n"                      ;
                             if ( this->numrec <  10 )
                                {
                                  this->numrec  = 10                     ;
                                  cout << "/N parm too small. 10 used\n" ;
                                }
                             break                                       ;

             case    's'  :      // Seed for filling (0=spaces)
             case    'S'  :

                             if ( getnumb(p) <= MAXUHALFWORD )
                                  this->seed   = getnumb(p)   ;
                                  else
                                  cout << "/S parm too big. "
                                       << this->seed
                                       << " used\n"           ;
                             break                            ;

             default :
                        break  ;       // unknown parm, so ignore it
           } ;

        } ;

    return ;

 }

//
// The getnumb private function within the parms class
//     merely extracts the numeric part of a /Pnnnn
//
FULLWORD parms::getnumb( char * const ptr )
 {
    FULLWORD   numb = 0   ;
    char     * p    = 0   ;
    char       c    = ' ' ;

    for ( p = ptr ; *p != '\0' ; p++ )
        {
          c = *p ;
          if ( c == ' ' ) break ;
          if ( ( c >= '0' ) && ( c <= '9' ) )
                numb = ( numb * 10 ) + ( c - '0' ) ;
        }
    return ( numb ) ;
  }

//
// Main procedure - generate a File filled with data
//
int  main ( int     const argc,
            char ** const argv,
            char ** const envp  )
{
 parms       myparms        ;
 HALFWORD    lrecl    = 0   ;
 HALFWORD    numrec   = 0   ;
 HALFWORD    seed     = 0   ;
 HALFWORD    currec   = 0   ;
 HALFWORD    i        = 0   ;
 char        curfill  = ' ' ;
 char      * buffer   = 0   ;
 char      * p        = 0   ;
 FILE      * fhandle  = 0   ;
 char        filename??(??) = "12345678.123" ;


 myparms.parmset(argc, argv, envp)      ;  //set the parms
 lrecl  = myparms.getlrecl ()           ;
 numrec = myparms.getnumrec()           ;
 seed   = myparms.getseed  ()           ;
          myparms.getfilename(filename) ;

                                    //Display program parms
 cout << "Program parms are: "                           ;
 for ( int j=0 ; j<argc ; cout << argv??(j++??) << " " ) ;
 cout << endl                                            ;


 if ( argc == 1 )                    //Show parms ONLY if none given
    {
      cout << "\nMQODG will generate a file of a specified size\n\n"
           << "Parms for MQODG are /F<file name>\n"
           << "                    /L<LRECL>         /N<number of records>\n"
           << "                    /S<fill>\n\n"
           << "/S0 space fill the record    otherwise random digit fill\n\n"
           ;
      return (0) ;
    }

 cout << "Generating file " << filename << "\nwhich will contain "
      << numrec << " records of " << lrecl << " bytes\n"
      << "random filling is " << (( seed == 0 ) ? "disabled" : "enabled") << endl ;

 if ( seed != 0 ) srand(seed) ;            //set random fill seed

 buffer = (char *) malloc(lrecl+2) ;       //get generation buffer
 if ( buffer == 0 )
   {
     cout << "Getmain of Buffer failed, Program ending" << endl ;
     return (4) ;
   }

 fhandle = fopen(filename,"w, recfm=f") ;  //open the output file
 if ( fhandle == 0 )
    {
     cout << "Open for output of failed. Program ending" << endl ;
     return (8) ;
    }

                                          // Build the file records of format
                                          // <numrec,lrecl>....fill...*

 for ( currec=0 ; currec < numrec ; currec++ )
     {
      sprintf(buffer,"<%06.6d.%06.6d>",currec+1,lrecl) ;    //set the known prefix
                                                            //fill the rest with spaces or
                                                            //random numbers
      for ( i=0 , p = buffer+15 ; i < (lrecl-15-1) ; i++, p++ )
          {
            if ( seed == 0 ) curfill = ' ' ;
                 else
                             curfill = (rand() % 10) + '0' ;
            *p = curfill  ;
          }
      *(p+0) = '*'  ;                                       //set the end-of-record *
      *(p+1) = '\n' ;                                       //marker and write the
      *(p+2) = '\0' ;                                       //record
      if ( fputs(buffer,fhandle) == EOF )
         {
           cout << "File output failed. Program ending" << endl ;
           return(12) ;
         }

     } //End of file generation loop

 if ( fclose(fhandle) == EOF )                              //Close the file
    {
      cout << "File output failed. Program ending" << endl ;
      return(16) ;
    }

 free(buffer)    ;                                          //Release the buffer
                                                            //and end
 cout << "File " << filename << " successfully generated" << endl ;

 return(0)       ;

}

// End of MQODG.CPP source
