// MQNFLTOQ.CPP program

//*********************************************************************************************
//*                                                                                           *
//*                            MQNQFLTOQ.CPP program                                          *
//*                            =====================                                          *
//*                                                                                           *
//*   This program copy a file to a Queue, adding a Zero-Length record aresiding              *
//*                                                                                           *
//*   The Zero-Length record will act as an End of Queue Indication.                          *
//*                                                                                           *
//*                                                                                           *
//*   The parameters are  /Maaaaaa    -> the name of the Queue Manager                        *
//*                                                  default -> blanks (default QM)           *
//*                                                                                           *
//*                       /Qaaaaaa    -> the name of the Queue                                *
//*                                                  default -> blanks                        *
//*                                                                                           *
//*                       /Faaaaaa    -> the name of the file to copy to the Queue            *
//*                                                  default -> blanks                        *
//*                                                                                           *
//*                       /Snnnnnn    -> the Syncpoint Interval                               *
//*                                       /S-     -> Outside of Unit of Work                  *
//*                                       /S0     -> Inside UOW, but Syncpoint at end         *
//*                                       /Snnnn  ->           , Syncpoint after every n mess *
//*                                                                                           *
//*                       /Pd         -> Control the persistance of the messages. /P+ forces  *
//*                       /P+            Persistant Messages, /P- forces non-Persistance, and *
//*                       /P-            /Pd uses the default Persistance from the Queue      *
//*                                                                                           *
//*                       /V+         -> Write Diagnotic Messages (default)                   *
//*                       /V-         -> Don't display any messages                           *
//*                                                                                           *
//*                       /Z+         -> Write A Zero-Length record on EOF (default)          *
//*                       /Z-         -> Don't write this record                              *
//*                                                                                           *
//*                                                                                           *
//*   Example : MQNFLTOQ /MRAHQM1 /QRAH1 /Fsource.1 /S0 /PD  will copy the file 'source.1'    *
//*                                                          into the queue RAH1 owned by     *
//*                                                          Queue Manager RAHQM1, within     *
//*                                                          a Single Unit of Work, with the  *
//*                                                          Persistance of the messages      *
//*                                                          defined by the Queue, together   *
//*                                                          with a Zero-Length record to     *
//*                                                          show the end-of-queue.           *
//*                                                                                           *
//*   Standard Messages are written to show information if /V+ is specified.                  *
//*                                                                                           *
//*                                                                                           *
//*                                                                                           *
//*   Return Codes :    +ve  -> Value of desired Attribute                                    *
//*                     -ve  -> Reason Code for a failure                                     *
//*                             However, a large attribute value >64k will also come          *
//*                                      out negative. Thus, treat only -ve numbers around    *
//*                                      the 2000 mark as Error codes.                        *
//*                                                                                           *
//*                                                                                           *
//*   Copyright : This program is Copyrighted by IBM UK Labs Limited, 1995.                   *
//*                                                                                           *
//*                                                                                           *
//*   Support   : This program is not supported in any way by IBM. It is distributed          *
//*               only to show techniques for Message Queueing. It may be freely              *
//*               modified and adapted by your installation.                                  *
//*                                                                                           *
//*   Author    : Robert Harris,                                                              *
//*               Object Technology Products,                                                 *
//*               IBM Hursley Park,                                                           *
//*               England.                                                                    *
//*                                                                                           *
//*********************************************************************************************

//
// System Includes
//

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <time.h>

//
// MQ Includes
//

#include <cmqc.h>

//
// Windows includes
//

#include <windows.h>

//
// Definitions of datatypes
//

#define FULLWORD     long
#define HALFWORD     int
#define MAXSFULLWORD (68719476735)
#define MAXUFULLWORD (4294967295)
#define MAXSHALFWORD (4095)
#define MAXUHALFWORD (65535)
#define MAXFILENAME  (255)
#define TRUE         1
#define FALSE        0

//
// The parms class contains the unravelling of the
//     supplied parameters
//

class parms
{
   private:

       FULLWORD    verbose                         ; // V flag 0=- 1=+
       FULLWORD    persist                         ; // P flag 0=default 1=Pers 2=NotP
       FULLWORD    syncint                         ; // S flag -1=outside 0=end n=interval
       FULLWORD    zrl                             ; // Z flag 0=- 1=+
       char        qm ??(MQ_Q_MGR_NAME_LENGTH+1??) ; // QM    name
       char        q  ??(MQ_Q_NAME_LENGTH+1    ??) ; // Q     name
       char        fn ??(MAXFILENAME+1         ??) ; // File  name

       FULLWORD parms::getnumb( char * const ptr ) ;    //Private func to get nums out of parms


   public:
                 parms       () { verbose       =  1       ; //Constructor
                                  persist       =  0       ; //  defaults
                                  syncint       =  0       ; //
                                  zrl           =  1       ; //
                                  memset(qm,0,sizeof(qm))  ; //
                                  memset(q ,0,sizeof(q ))  ; //
                                  memset(fn,0,sizeof(fn))  ; //
                                }

       FULLWORD  isverbose   () { if ( verbose == 1 ) return(TRUE)  ;   //Access func to the pa
                                       else           return(FALSE) ; } ;

       FULLWORD  getsyncint  () { return (syncint) ; } ;


       FULLWORD  getpersist  () { return (persist) ; } ;

       FULLWORD  getzrl      () { return (zrl    ) ; } ;

       char    * getqmname   (char * const dest)
                             {
                               strcpy(dest,qm) ;
                               return(dest)    ;
                             }

       char    * getqname    (char * const dest)
                             {
                               strcpy(dest,q)  ;
                               return(dest)    ;
                             }

       char    * getflname   (char * const dest)
                             {
                               strcpy(dest,fn) ;
                               return(dest)    ;
                             }


       void     parmset ( int     const argc,                       //Function to set
                          char ** const argv,                       // up parms in the
                          char ** const envp  ) ;                   // class


} ;

//
// The parmset function is the part of the parms class
//     that builds the parameters
//
// Valid parms are :
//
//            /Maaaaa   -> The Queue Manager Name
//            /Qaaaaa   -> The Queue Name
//            /Faaaaa   -> The File  Name
//            /P d + -  -> The Required Message Persistance  (Q,Y,N)
//            /S - 0 n  -> The Syncpoint Interval (Outside,end,int)
//            /V + -    -> Message Displaying (Y,N)
//            /Z + -    -> Zero Length Record (Y,N)
//
void parms::parmset ( int     const argc,
                      char ** const argv,
                      char ** const envp  )
 {
    int     i = 0   ;
    char  * p = 0   ;
    char    c = ' ' ;
    char    s = ' ' ;
    char    v = ' ' ;

    for ( i=1 ; i < argc ; i++ )     // loop through all the parms
        {
          p = argv??(i??) ;          // start of current parm
          c = *p          ;          // should be the selector, so ignore it
          s = *(++p)      ;          // the parm key
          p++             ;          // start of parm data
          v = *p          ;          // first of the parm data

          switch ( s )
           {
             case    'M'  :      // Queue Manager
             case    'm'  :

                             strncpy(this->qm,p,MQ_Q_MGR_NAME_LENGTH) ;
                             break                                    ;

             case    'Q'  :      // Queue
             case    'q'  :

                             strncpy(this->q,p,MQ_Q_NAME_LENGTH) ;
                             break                               ;
             case    'f'  :      // Queue
             case    'F'  :

                             strncpy(this->fn,p,MAXFILENAME) ;
                             break                           ;

             case    'p'  :      // Persistance required
             case    'P'  :

                             switch ( v )
                                {
                                  case 'd'  :     //Take from Q
                                  case 'D'  :
                                              persist = 0 ;
                                              break       ;

                                  case '+'  :     //Force Persistance
                                              persist = 1 ;
                                              break       ;

                                  case '-'  :     //Force NotPersistance
                                              persist = 2 ;
                                              break       ;

                                  default   : break       ;
                                }
                             break ;

             case    's'  :      // Syncpoint Interval
             case    'S'  :

                             switch ( v )
                                {
                                  case '-'  :     //Outside UOW
                                              syncint = -1         ;
                                              break                ;

                                  default   :     //End of every n
                                              syncint = getnumb(p) ;
                                              break                ;
                                }
                             break ;

             case    'v'  :      // Message usage
             case    'V'  :

                             switch ( v )
                                {

                                  case '+'  :     //Yes
                                              verbose = 1 ;
                                              break       ;

                                  case '-'  :     //No
                                              verbose = 0 ;
                                              break       ;

                                  default   : break       ;
                                }
                             break ;

             case    'z'  :      // Zero Length Record
             case    'Z'  :

                             switch ( v )
                                {

                                  case '+'  :     //Yes
                                              zrl   = 1 ;
                                              break     ;

                                  case '-'  :     //No
                                              zrl   = 0 ;
                                              break     ;

                                  default   : break     ;
                                }
                             break ;


             default :
                        break  ;       // unknown parm, so ignore it
           } ;

        } ;

    return ;

 }

//
// The getnumb private function within the parms class
//     merely extracts the numeric part of a /Pnnnn
//
FULLWORD parms::getnumb( char * const ptr )
 {
    FULLWORD   numb = 0   ;
    char     * p    = 0   ;
    char       c    = ' ' ;

    for ( p = ptr ; *p != '\0' ; p++ )
        {
          c = *p ;
          if ( c == ' ' ) break ;
          if ( ( c >= '0' ) && ( c <= '9' ) )
                numb = ( numb * 10 ) + ( c - '0' ) ;
        }
    return ( numb ) ;
  }



//
// RAHMQN Class Library
//

class rahmqn
 {
   public :

    char        qm  ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name
    char        q   ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name
    MQHCONN     qmh                              ; //QM handle
    MQHOBJ      qh                               ; //Q  handle
    MQLONG      rc                               ; //Return Code
    MQLONG      ac                               ; //Reason Code
    MQLONG      lastrc                           ; //Prev Return Code
    MQLONG      lastac                           ; //Prev Reason Code


    rahmqn ( void )               ;  //Contructor for new  use of QM
    rahmqn ( MQHCONN const uqmh)  ;  //Contructor for same use of QM

    MQLONG      conn ( void )               ;  //Connect    to QM
    MQLONG      conn ( char * const uqm )   ;  //  with QM name supplied

    MQLONG      disc ( void )               ;  //Disconnect from QM

    void        cleanup ( void )            ;  //Close and Disconnect

    MQOD        odopen                      ;  //OD for MQOPEN usage
    MQLONG      open (MQLONG const uopts )  ;  //MQOPEN call
    MQLONG      open (char * const uq    ,     //  with Q name supplied
                      MQLONG const uopts )  ;  //
    MQLONG      open (MQOD * const uod   ,     //  with OD and options
                      MQLONG const uopts )  ;  //

    MQLONG      close (MQLONG const uopts ) ;  //MQCLOSE call

    MQLONG      commit   ( void ) ; //Syncpoint UOW
    MQLONG      rollback ( void ) ; //Backout   UOW

    MQLONG      inq ( MQLONG   const  uselnum ,             //MQINQ call
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  ) ;

    MQLONG      set ( MQLONG   const uselnum  ,             //MQSET call
                      MQLONG         usels    ,
                      MQLONG   const uinum    ,
                      MQLONG *       uiattr   ,
                      MQLONG   const ucharl   ,
                      char   *       ucattr   ) ;

    MQMD        pmd                             ;           //PUT Message Dwescriptor
    MQLONG      put ( MQPMO  * const  uopts   ,             //MQPUT call
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

    MQLONG      put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

                                                            //PUT Zero-Length record
    MQLONG      putz ( MQPMO  * const  uopts     ) ;        //No Tag, Use Instance MD

    MQLONG      putz ( MQMD   *        upmd      ,          //No Tag, use supplied MD
                       MQPMO  * const  uopts     ) ;

    MQLONG      putz ( MQPMO    * const  uopts   ,          //Tag, Use Instance MD
                       MQBYTE24 * const  utag    ) ;

    MQLONG      putz ( MQMD     *        upmd    ,          //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    ) ;

    MQMD        bmd                                  ;      //Browse Message Descriptor
    MQGMO       bmo                                  ;      //Browse MQGET options
    MQLONG      browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;

    MQLONG      browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;


    MQMD        gmd                               ;         //GET Message Descriptor
    MQLONG      get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;

    MQLONG      get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;


    double      gettimeconn     () { return (time_conn     ) ; }
    double      gettimedisc     () { return (time_disc     ) ; }
    double      gettimeopen     () { return (time_open     ) ; }
    double      gettimeclose    () { return (time_close    ) ; }
    double      gettimeinq      () { return (time_inq      ) ; }
    double      gettimeset      () { return (time_set      ) ; }
    double      gettimeput      () { return (time_put      ) ; }
    double      gettimeget      () { return (time_get      ) ; }
    double      gettimecommit   () { return (time_commit   ) ; }
    double      gettimerollback () { return (time_rollback ) ; }
    double      getcputconn     () { return (cput_conn     / CLOCKS_PER_SEC) ; }
    double      getcputdisc     () { return (cput_disc     / CLOCKS_PER_SEC) ; }
    double      getcputopen     () { return (cput_open     / CLOCKS_PER_SEC) ; }
    double      getcputclose    () { return (cput_close    / CLOCKS_PER_SEC) ; }
    double      getcputinq      () { return (cput_inq      / CLOCKS_PER_SEC) ; }
    double      getcputset      () { return (cput_set      / CLOCKS_PER_SEC) ; }
    double      getcputput      () { return (cput_put      / CLOCKS_PER_SEC) ; }
    double      getcputget      () { return (cput_get      / CLOCKS_PER_SEC) ; }
    double      getcputcommit   () { return (cput_commit   / CLOCKS_PER_SEC) ; }
    double      getcputrollback () { return (cput_rollback / CLOCKS_PER_SEC) ; }

 private :

    double      time_conn      ;
    double      time_disc      ;
    double      time_open      ;
    double      time_close     ;
    double      time_inq       ;
    double      time_set       ;
    double      time_put       ;
    double      time_get       ;
    double      time_commit    ;
    double      time_rollback  ;


    double      cput_conn      ;
    double      cput_disc      ;
    double      cput_open      ;
    double      cput_close     ;
    double      cput_inq       ;
    double      cput_set       ;
    double      cput_put       ;
    double      cput_get       ;
    double      cput_commit    ;
    double      cput_rollback  ;
 } ;

//
// rahmqn member function definitions
//

//
// Constructor functions
//
//     without a parm, then this is a new use of a QM
//     with          , then this is an additional access on a
//                     previously connected QM
//

rahmqn::rahmqn ( void )
{
  memset(qm,0,sizeof(qm)) ;
  memset(q ,0,sizeof(q )) ;
  qmh    = 0              ;
  qh     = 0              ;
  rc     = 0              ;
  ac     = 0              ;
  lastrc = 0              ;
  lastac = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

rahmqn::rahmqn ( MQHCONN const uqmh )
{
  memset(qm,0,sizeof(qm)) ;
  memset(q ,0,sizeof(q )) ;
  qmh    = uqmh           ;
  qh     = 0              ;
  rc     = 0              ;
  ac     = 0              ;
  lastrc = 0              ;
  lastac = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

//
// rahmqn destructor functions
//
//   these are not provided
//
//    (Note that a disc call removes access to ALL
//     instances of RAHMQN sharing the Handle. as I
//     am not aiming at a 'proper' C++ implementation
//     I have not provided code to cope with this event).
//

//
// rahmqn functions to access a Queue Manager
//

//
// conn  will connect to a Queue Manager
//       saving the QMH
//
//       The version with no parms will use the
//           pre-set Queue Manager name
//
//       The alternative access is to quote a
//           Queue Manager Name which is the
//           one to be used
//

MQLONG  rahmqn::conn ( char * const uqm )
{
  double c1, c2, t1, t2         ;
  strncpy(qm,uqm,sizeof(qm)-1)  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::conn ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// disc  will disconnect from a Queue Manager
//       deleting the QMH
//


MQLONG  rahmqn::disc ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQDISC ( &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_disc += 1.0000 * ( c2 - c1 )      ;
  time_disc += 1.0000 * difftime(t2,t1)  ;

  qmh = 0     ;

  return (ac) ;
}

//
// cleanup will Close the Q (if known) and Disconnect from
//         the QM (if known). This will effect other instances
//         which were created with a QM handle
//

void    rahmqn::cleanup ( void )
{
  double c1, c2, t1, t2         ;

  if ( qh  != 0 ) close(MQCO_NONE) ;
  if ( qmh != 0 ) disc ()          ;
  return                           ;
}

//
// commit will Syncpoint ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqn::commit ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCMIT ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_commit += 1.0000 * ( c2 - c1 )      ;
  time_commit += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// rollback will backout ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqn::rollback ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQBACK ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_rollback += 1.0000 * ( c2 - c1 )      ;
  time_rollback += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// open  will provide access to a Queue
//
//       The OPEN options are always supplied
//           by the caller.
//
//       If the version without a Queue Name is
//          used, then the Name from the Instance
//          is taken.
//

MQLONG  rahmqn::open ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqn::open ( char * const uq    ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  strncpy(q,uq,sizeof(q)-1)   ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqn::open ( MQOD * const uod   ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memcpy(&odopen,uod,sizeof(odopen)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  memcpy(uod,&odopen,sizeof(odopen))     ;
  return (ac) ;
}

//
// close will stop access to a Queue
//
//       The CLOSE options are always supplied
//           by the caller.
//

MQLONG  rahmqn::close ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCLOSE ( qmh, &qh, uopts, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_close += 1.0000 * ( c2 - c1 )      ;
  time_close += 1.0000 * difftime(t2,t1)  ;

  qh = 0      ;

  return (ac) ;
}

//
// inq   will inquire upon the Attributes of
//       a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqn::inq ( MQLONG   const  uselnum ,
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQINQ ( qmh, qh, uselnum, &usels ,
                   uinum  , uiattr ,
                   ucharl , ucattr ,
                                     &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_inq += 1.0000 * ( c2 - c1 )      ;
  time_inq += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}
//
// set   will set the Attributes of a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqn::set ( MQLONG   const   uselnum ,
                      MQLONG           usels   ,
                      MQLONG   const   uinum   ,
                      MQLONG *         uiattr  ,
                      MQLONG   const   ucharl  ,
                      char   *         ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQSET ( qmh, qh, uselnum , &usels  ,
                   uinum   , uiattr  ,
                   ucharl  , ucattr  ,
                                       &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_set += 1.0000 * ( c2 - c1 )      ;
  time_set += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// put       will write a Message into a Queue
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqn::put ( MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// putz      will write a Zero-Length Message into a Queue
//
//           If a Tag is supplied, then this is used as the Message Descriptor
//              this must be an area of MQBYTE24 (24 bytes long)
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqn::putz ( MQPMO  * const  uopts )               //No Tag, Use Instance MD
{
  double c1, c2, t1, t2         ;

  memset(&this->pmd.MsgId,0,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::putz ( MQMD   *        upmd    ,              //No Tag, use supplied MD
                       MQPMO  * const  uopts   )
{
  double c1, c2, t1, t2         ;

  memset(upmd->MsgId,0,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqn::putz ( MQPMO    * const  uopts   ,            //Tag, Use Instance MD
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(&this->pmd.MsgId,utag,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::putz ( MQMD     *        upmd    ,            //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(upmd->MsgId,utag,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
//    Browse    will non-destructively read a Queue
//
//              Versions are supplied that use an
//              instance Message dewscriptor, or
//              one supplied by the caller
//

MQLONG  rahmqn::browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_FAIL_IF_QUIESCING               ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  memset(&bmd,0,sizeof(bmd))                                 ;
  memcpy(&bmd.StrucId,MQMD_STRUC_ID,sizeof(bmd.StrucId))     ;
  bmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &bmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_CONVERT+MQGMO_FAIL_IF_QUIESCING ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ubmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  bmo.Version        = MQMD_VERSION_1                        ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  gmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &gmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ugmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// End of RAHMQN Class definition and functions
//




//
// Main procedure - Write a File to a Queue
//
int  main ( int     const argc,
            char ** const argv,
            char ** const envp  )
{
 parms       myparms        ;
 rahmqn      mymq           ;

 char        qm  ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name
 char        q   ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name
 char        fn  ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name

 FULLWORD    syncint = 0    ;
 FULLWORD    persist = 0    ;
 FULLWORD    zrl     = 1    ;
 MQLONG      opts    = 0    ;
 MQPMO       pmo            ;

 MQLONG      maxmsgl = 0    ;
 char     *  buffer  = 0    ;
 FILE     *  fhandle = 0    ;
 FULLWORD    lastpos = 0    ;
 FULLWORD    curpos  = 0    ;

 double      totcput = 0    ;
 double      tottime = 0    ;
 double      totc1   = 0    ;
 double      totc2   = 0    ;
 double      tote1   = 0    ;
 double      tote2   = 0    ;

 MQLONG      mqrc    = 0    ;
 MQLONG      mqac    = 0    ;
 MQLONG      ac      = 0    ;
 MQLONG      rc      = 0    ;

 FULLWORD    syncno  = 0    ;
 FULLWORD    totsync = 0    ;
 FULLWORD    recno   = 0    ;

 totc1 = clock()    ;                      //Set Timers
 tote1 = time(NULL) ;

 myparms.parmset(argc, argv, envp)      ;  //set the parms

 myparms.getqmname(qm)          ;    //Get the data
 myparms.getqname (q )          ;    // from the
 myparms.getflname(fn)          ;    // parms
 syncint = myparms.getsyncint() ;    //
 persist = myparms.getpersist() ;    //
 zrl     = myparms.getzrl()     ;    //

 if ( myparms.isverbose() )          //Display program parms
    {
      cout << "Program parms are: " ;
      for ( int i=0 ; i<argc ; cout << argv??(i++??) << " " ) ;
      cout << endl ;
    }

 if ( argc == 1 )                    //Show parms ONLY if none given
    {
      cout << "\nMQNFLTOQ will copy a file to a queue \n\n"
           << "Parms for MQNFLTOQ are /M<queue manager>  /Q<queue> \n"
           << "                       /F<file name>      /P<persistance> \n"
           << "                       /S<sync interval>  /Z<ZeroLenRec>\n"
           << "                       /V<verbose> \n\n"
           << " /Pd from Qdef         /P+ Force Persistance   /P- Force Non-Persistance \n"
           << " /S- Run outside UOW   /S0 Sync at end         /Sn Sync every n \n"
           << " /Z+ Write ZeroLenRec  /Z- No ZRL at end of Q \n"
           << " /V+ display           /V- no messages \n\n"
           ;
      return (0) ;
    }


  if ( myparms.isverbose() ) switch ( persist )        //Display Message Persistance
   {
     case 0 :
              cout << "Message Persistance taken from Q definition" << endl ;
              break                                                         ;

     case 1 :
              cout << "Messages forced to be Persistant" << endl ;
              break                                              ;

     case 2 :
              cout << "Messages forced to be  Non-Persistant" << endl ;
              break                                                   ;

     default:
              break ;
   }

  if ( myparms.isverbose() ) switch ( syncint )        //Display UoW info
   {
     case -1:
              cout << "Messages to be written Outside of a UoW" << endl ;
              break                                                     ;

     case 0 :
              cout << "Messages(within a UoW) are Syncpointed after all have been added to the Q" << endl ;
              break                                                                                       ;

     default:
              cout << "Messages(within a UoW) are Syncpointed every " << syncint << " messages" << endl ;
              break                                                                                     ;
   }

  if ( myparms.isverbose() ) switch ( zrl )            //Display Zero Len Record
   {
     case 0 :
              cout << "A Zero Length Record will not be written" << endl ;
              break                                                                                       ;

     default:
              cout << "A Zero Length Record will be written" << endl ;
              break                                                                                     ;
   }

 switch ( syncint )              //Set Put options
  {
    case -1 :
               opts  = MQPMO_NO_SYNCPOINT      +
                       MQPMO_NO_CONTEXT        +
                       MQPMO_FAIL_IF_QUIESCING ;
               break                           ;

    default :
               opts  = MQPMO_SYNCPOINT         +
                       MQPMO_NO_CONTEXT        +
                       MQPMO_FAIL_IF_QUIESCING ;
               break                           ;
  }

 if ( rc == 0 )                      // Connect to QM
    {
      if ( myparms.isverbose() ) cout << "Connecting to QM " << qm << " " ;
      ac   = mymq.conn(qm)   ;
      mqrc = mymq.lastrc     ;
      mqac = mymq.lastrc     ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Open the Q for INQ/Put access
    {
      if ( myparms.isverbose() ) cout << "Opening Q " << q << " for Output " ;
      ac   = mymq.open(q,(MQOO_INQUIRE+MQOO_OUTPUT+MQOO_FAIL_IF_QUIESCING)) ;
      mqrc = mymq.lastrc     ;
      mqac = mymq.lastrc     ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Get MaxMessage Size
    {
      MQLONG   tselcount        = 1                   ;
      MQLONG   tsels            = MQIA_MAX_MSG_LENGTH ;
      MQLONG   tintcount        = 1                   ;
      MQLONG   tintattr         = 0                   ;
      MQLONG   tcharlen         = 0                   ;
      char     tcharattr        = '\0'                ;

      if ( myparms.isverbose() ) cout << "Obtaining Q Max Message Length " ;
      ac   =  mymq.inq(tselcount, tsels,
                                         tintcount, &tintattr  ,
                                         tcharlen , &tcharattr ) ;

      mqrc = mymq.lastrc          ;
      mqac = mymq.lastrc          ;

      maxmsgl = tintattr          ;

//    if ( ac != 0 ) rc = ac      ;    // This only works for a local Queue
                                       //  So to avoid the indignity of
                                       //  wandering down all the alias
                                       //  definitions, and so doing lots of
                                       //  MQOPENs for Inq access, I simply
                                       //  fiddle the length for Alias and
                                       //  Remote Queues
      if ( ac != 0 ) maxmsgl = MAXUHALFWORD ;

      if ( myparms.isverbose() ) cout << "RC=" << rc << " Max Len is " << maxmsgl << endl ;

      if ( maxmsgl > MAXUHALFWORD )
         {
           maxmsgl = MAXUHALFWORD ;
           if ( myparms.isverbose() ) cout << "which is forced to max supported size of " << MAXUHALFWORD << endl ;
         }
    }

 if ( rc == 0 )                      // Getmain the Buffer (always max file size)
    {
      if ( myparms.isverbose() ) cout << "Getmaining the File/Q Buffer " ;
      buffer = (char *)malloc(MAXUHALFWORD) ;
      if ( buffer == 0 ) rc = -8    ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Open the input file
    {
      if ( myparms.isverbose() ) cout << "Opening file " << fn << " " ;
      fhandle = fopen(fn,"rb")        ;
      if ( fhandle == NULL ) rc = -9  ;
      if ( myparms.isverbose() ) cout << ((rc != 0) ? "Operation Failed"
              : "File now Open for Input. File Read+Q Put starting" ) << endl ;
    }


 if ( rc == 0 )                      // Read File and Write Q
    {
      for ( recno = 0 , syncno = 1 ; ; recno++, syncno++ )
          {
            MQLONG   i      = 0   ;
            char     exiter = 'N' ;
            char   * c      = 0   ;
            char   * p      = 0   ;
            MQLONG   frc    = 0   ;
            FULLWORD reclen = 0   ;

            memset(&pmo,0,sizeof(pmo))  ;               //Set Put Message Options
            memcpy(&pmo.StrucId,MQPMO_STRUC_ID,sizeof(pmo.StrucId))  ;
            pmo.Version        = MQPMO_VERSION_1                     ;
            pmo.Options        = opts                                ;

            memset(&mymq.pmd,0,sizeof(mymq.pmd))  ;     //Set Put Message Desc
            memcpy(&mymq.pmd.StrucId,MQMD_STRUC_ID,sizeof(mymq.pmd.StrucId)) ;
            memcpy(&mymq.pmd.Format ,MQFMT_NONE   ,sizeof(mymq.pmd.Format )) ;
            mymq.pmd.Version        = MQMD_VERSION_1            ;
            mymq.pmd.Report         = MQRO_NONE                 ;
            mymq.pmd.MsgType        = MQMT_DATAGRAM             ;
            mymq.pmd.Expiry         = MQEI_UNLIMITED            ;
            mymq.pmd.Feedback       = MQFB_NONE                 ;
            mymq.pmd.Encoding       = MQENC_NATIVE              ;
            mymq.pmd.CodedCharSetId = MQCCSI_Q_MGR              ;
            mymq.pmd.Priority       = MQPRI_PRIORITY_AS_Q_DEF   ;
            mymq.pmd.PutApplType    = MQAT_OS2                  ;
            switch ( persist )
             {
               case 0 :
                        mymq.pmd.Persistence = MQPER_PERSISTENCE_AS_Q_DEF ;
                        break                                             ;

               case 1 :
                        mymq.pmd.Persistence = MQPER_PERSISTENT           ;
                        break                                             ;

               case 2 :
                        mymq.pmd.Persistence = MQPER_NOT_PERSISTENT       ;
                        break                                             ;

               default:
                        break                                             ;
             }

             memset(buffer,0,MAXUHALFWORD) ;  //Clear the File Buffer
             reclen  = 0                   ;  //and reset record
             lastpos = curpos              ;  //indicators
             exiter  = 'R'                 ;

                                              //Read the file
                                              //until an error, EOF,
                                              //or a \n is found


            c = fgets(buffer,MAXUHALFWORD,fhandle) ;
            curpos = ftell(fhandle)                ;
            reclen = curpos - lastpos              ;

            if ( reclen > maxmsgl )
               {                //File record too big for Q
                 exiter = 'B' ;
               }

            if ( ( c == NULL ) && (ferror(fhandle)) )
               {                //Error
                  exiter = 'E' ;
               }

            if ( ( c == NULL ) && ( ??- ferror(fhandle)) )
               {                //EOF
                  exiter = 'Y' ;
               }


            switch ( exiter )         //Decide what to do with record
              {
                case 'E'  :           //File Error
                             frc = -10 ;
                             if ( myparms.isverbose() ) cout << "File error on record " << recno+1 << endl ;
                             break     ;

                case 'B'  :           //File Record too big
                             frc = -11 ;
                             if ( myparms.isverbose() ) cout << "File record size of " << reclen <<
                                        " > Q Max Message Len of " << maxmsgl << " on record " << recno+1 << endl ;
                             break     ;


                case 'Y'  :           //End of File
                             frc = 0   ;
                             break     ;


                case 'R'  :           //End of record found (-2 to remove CRLF at end of record)
                             frc = mymq.put ( &pmo, reclen-2, buffer ) ;
                             if ( frc != 0 )
                                {
                                  rc = frc ;
                                  if ( myparms.isverbose() )
                                      cout << "PUT error on record " << recno << " RC= "
                                           << frc << endl ;
                                }
                             break     ;


                default   :           //Bad Exiter value
                             frc = -12 ;
                             break     ;

              } //End of exiter SWITCH block

            if ( ( exiter == 'Y' ) && ( rc == 0 )  )  // Successful load into Q
               {
                 if ( myparms.isverbose() ) cout << "All file records placed on Q" << endl ;
                 if ( zrl != 0 ) // Write the Zero Length Record if required (EoQ)
                    {
                      if ( myparms.isverbose() ) cout << "Now writing the Zero Length Record " ;
                      memset(&pmo,0,sizeof(pmo))  ;               //Set Put Message Options
                      memcpy(&pmo.StrucId,MQPMO_STRUC_ID,sizeof(pmo.StrucId))  ;
                      pmo.Version        = MQPMO_VERSION_1                     ;
                      pmo.Options        = opts                                ;

                      memset(&mymq.pmd,0,sizeof(mymq.pmd))  ;     //Set Put Message Desc
                      memset(&mymq.pmd,0,sizeof(mymq.pmd))  ;
                      memcpy(&mymq.pmd.StrucId,MQMD_STRUC_ID,sizeof(mymq.pmd.StrucId)) ;
                      memcpy(&mymq.pmd.Format ,MQFMT_NONE   ,sizeof(mymq.pmd.Format )) ;
                      mymq.pmd.Version        = MQMD_VERSION_1            ;
                      mymq.pmd.Report         = MQRO_NONE                 ;
                      mymq.pmd.MsgType        = MQMT_DATAGRAM             ;
                      mymq.pmd.Expiry         = MQEI_UNLIMITED            ;
                      mymq.pmd.Feedback       = MQFB_NONE                 ;
                      mymq.pmd.Encoding       = MQENC_NATIVE              ;
                      mymq.pmd.CodedCharSetId = MQCCSI_Q_MGR              ;
                      mymq.pmd.Priority       = MQPRI_PRIORITY_AS_Q_DEF   ;
                      mymq.pmd.PutApplType    = MQAT_OS2                  ;
                      switch ( persist )
                       {
                          case 0 :
                                   mymq.pmd.Persistence = MQPER_PERSISTENCE_AS_Q_DEF ;
                                   break                                             ;

                          case 1 :
                                   mymq.pmd.Persistence = MQPER_PERSISTENT           ;
                                   break                                             ;

                          case 2 :
                                   mymq.pmd.Persistence = MQPER_NOT_PERSISTENT       ;
                                   break                                             ;

                          default:
                                   break                                             ;
                       }

                      ac   = mymq.putz(&pmo)        ;
                      mqrc = mymq.lastrc            ;
                      mqac = mymq.lastrc            ;

                      if ( ac != 0 ) rc = ac ;
                      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
                    }
               }  // End of Zero-Length record writing


                                              //Syncpoint if its
                                              //to be done after
                                              //each PUT, or if
                                              //the required number
                                              //of PUTS have been
                                              //done (or EOF hit)

            if ( ( rc == 0 ) && ( syncint >= 0 ) )
               {
                 if (    ( ( syncint >= 1       ) && ( syncno  >= syncint ) )
                      || (   exiter  == 'Y'     ) )
                    {
                      if ( ( exiter == 'Y' ) && ( myparms.isverbose() ) )
                           cout << "Doing the Final Syncpoint " ;

                      syncno = 0          ;
                      totsync++           ;
                      frc = mymq.commit() ;

                      if ( ( exiter == 'Y' ) && ( myparms.isverbose() ) )
                           cout << " RC=" << frc << endl ;

                      if ( frc != 0 )
                           {
                           if ( myparms.isverbose() ) cout << "Syncpoint Error RC=" << frc << endl ;
                         }
                    }
               } // End of Syncpoint Block

                                              //If there was a
                                              //failure of any sort
                                              //whilst running
                                              //inside a Uow, then
                                              //Backout any updates

            if ( ( frc != 0 ) && ( syncint != -1 ) )
               {
                 exiter = 'X'           ;
                 rc = frc               ;
                 if ( myparms.isverbose() ) cout << "Initiating Rollback due to RC " << rc << endl ;
                 ac = mymq.rollback() ;
                 if ( myparms.isverbose() ) cout << "Rollback RC=" << ac << endl ;
               } // End of Rollback Block

            rc = frc                   ; //Set return Code

            if ( exiter != 'R' ) break ; //Get next record
                                         //unless error occured

        } // End of Record Processing FOR loop

    } // End of File/Queue Processing Block


                                                //End of processing

 if ( rc == 0 )                      // Close the Q
    {
      if ( myparms.isverbose() ) cout << "Closing the Q " ;
      ac   = mymq.close(MQCO_NONE)  ;
      mqrc = mymq.lastrc            ;
      mqac = mymq.lastrc            ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Disconnect from QM
    {
      if ( myparms.isverbose() ) cout << "Disconnecting from QM " ;
      ac   = mymq.disc()            ;
      mqrc = mymq.lastrc            ;
      mqac = mymq.lastrc            ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Close the File
    {
      if ( myparms.isverbose() ) cout << "Closing the file " ;
      ac  = fclose(fhandle)        ;

      if ( ac != 0 ) rc = -13 ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( ( rc == 0 ) && ( myparms.isverbose() ) )  //Issue Good completion message
    {

      totc2 = clock()    ;
      tote2 = time(NULL) ;
      totcput = 1.0000 * ( totc2 - totc1  ) / CLOCKS_PER_SEC   ;
      tottime = 1.0000 * difftime(tote2,tote1)                 ;

      cout << recno << " records written from file " << fn << " to Q " << q << "\n"       ;
      cout << "CPU time (ms) and Elapsed Time (s) per MQ verb follow:\n"                   ;
      cout << " >Connect    " << mymq.getcputconn     () << " " << mymq.gettimeconn     () << "\n" ;
      cout << " >Disconnect " << mymq.getcputdisc     () << " " << mymq.gettimedisc     () << "\n" ;
      cout << " >Open       " << mymq.getcputopen     () << " " << mymq.gettimeopen     () << "\n" ;
      cout << " >Close      " << mymq.getcputclose    () << " " << mymq.gettimeclose    () << "\n" ;
      if ( totsync == 0 )
           cout << " >Commit     " << mymq.getcputcommit   () << " " << mymq.gettimecommit   () << "\n" ;
           else
           cout << " >Commit     " << mymq.getcputcommit   () << " " << mymq.gettimecommit   () <<
                   " per call "    << mymq.getcputcommit   () / totsync <<
                   " "             << mymq.gettimecommit   () / totsync << "\n" ;
      cout << " >Rollback   " << mymq.getcputrollback () << " " << mymq.gettimerollback () << "\n" ;
      cout << " >Inquire    " << mymq.getcputinq      () << " " << mymq.gettimeinq      () << "\n" ;
      cout << " >Set        " << mymq.getcputset      () << " " << mymq.gettimeset      () << "\n" ;
      cout << " >Get        " <<    mymq.getcputget()               << " " << mymq.gettimeget() <<
              "  per record " <<  ( mymq.getcputget() / (recno+1) ) <<
                          " " <<  ( mymq.gettimeget() / (recno+1) ) << "\n" ;
      cout << " >Put        " <<    mymq.getcputput()               << " " << mymq.gettimeput() <<
              "  per record " <<  ( mymq.getcputput() / (recno+1) ) <<
                          " " <<  ( mymq.gettimeput() / (recno+1) ) << "\n" ;
      cout << "Total MQ CPU time " << (   mymq.getcputconn     ()
                                        + mymq.getcputdisc     ()
                                        + mymq.getcputopen     ()
                                        + mymq.getcputclose    ()
                                        + mymq.getcputcommit   ()
                                        + mymq.getcputrollback ()
                                        + mymq.getcputinq      ()
                                        + mymq.getcputset      ()
                                        + mymq.getcputget      ()
                                        + mymq.getcputput      () ) << "s "
           << "Total MQ Elapsed time " << (   mymq.gettimeconn     ()
                                            + mymq.gettimedisc     ()
                                            + mymq.gettimeopen     ()
                                            + mymq.gettimeclose    ()
                                            + mymq.gettimecommit   ()
                                            + mymq.gettimerollback ()
                                            + mymq.gettimeinq      ()
                                            + mymq.gettimeset      ()
                                            + mymq.gettimeget      ()
                                            + mymq.gettimeput      () )  << "s\n" ;
      cout << "Total CPU " << totcput << "s, Total elapsed time " << tottime << "s\n" ;

    }

 fclose(fhandle)                       ; //Close the file
 if ( buffer != 0 ) free(buffer)       ; //Freemain the buffer
 mymq.cleanup()                        ; //Stop all accesses

 return(rc) ;                           //Return RC

}

// End of MQNFLTOQ.CPP source
