// MQNQMTOQ.CPP program

//*********************************************************************************************
//*                                                                                           *
//*                            MQNQMTOQ.CPP program                                           *
//*                            ====================                                           *
//*                                                                                           *
//*   This program will MOVE a Queue to another Queue, stopping on the First                  *
//*                     Zero-Length record                                                    *
//*                                                                                           *
//*                                                                                           *
//*   The Zero-Length record will act as an End of Queue Indication.                          *
//*                                                                                           *
//*                                                                                           *
//*   The parameters are  /Maaaaaa    -> the name of the source Queue Manager                 *
//*                                                  default -> blanks (default QM)           *
//*                                                                                           *
//*                       /Qaaaaaa    -> the name of the source Queue                         *
//*                                                  default -> blanks                        *
//*                                                                                           *
//*                       /Naaaaaa    -> the name of the destination Queue Manager            *
//*                                                  default -> blanks (default QM)           *
//*                                                                                           *
//*                       /Raaaaaa    -> the name of the destination Queue                    *
//*                                                  default -> blanks                        *
//*                                                                                           *
//*                       /Snnnnnn    -> the Syncpoint Interval                               *
//*                                       /S-     -> Outside of Unit of Work                  *
//*                                       /S0     -> Inside UOW, but Syncpoint at end         *
//*                                       /Snnnn  ->           , Syncpoint after every n mess *
//*                                                                                           *
//*                       /Pd         -> Control the persistance of the messages. /P+ forces  *
//*                       /P+            Persistant Messages, /P- forces non-Persistance, and *
//*                       /P-            /Pd uses the default Persistance from the Queue      *
//*                                                                                           *
//*                       /W+         -> Wait for messages (default)                          *
//*                       /W-         -> Don't wait for messages                              *
//*                                                                                           *
//*                       /V+         -> Write Diagnotic Messages (default)                   *
//*                       /V-         -> Don't display any messages                           *
//*                                                                                           *
//*                                                                                           *
//*   Example : MQNQMTOQ /MRAH1 /NRAH1 /QP1 /RN1 /S-         will move the contents of Queue  *
//*                                                          P1 to Queue N1 outside of a UoW. *
//*                                                                                           *
//*   Standard Messages are written to show information if /V+ is specified.                  *
//*                                                                                           *
//*                                                                                           *
//*                                                                                           *
//*   Return Codes :    0    -> Operation succeeded                                           *
//*                     >0   -> Reason Code for a failure                                     *
//*                                                                                           *
//*                                                                                           *
//*   Copyright : This program is Copyrighted by IBM UK Labs Limited, 1994.                   *
//*                                                                                           *
//*                                                                                           *
//*   Support   : This program is not supported in any way by IBM. It is distributed          *
//*               only to show techniques for Message Queueing. It may be freely              *
//*               modified and adapted by your installation.                                  *
//*                                                                                           *
//*   Author    : Robert Harris,                                                              *
//*               Object Technology Products,                                                 *
//*               IBM Hursley Park,                                                           *
//*               England.                                                                    *
//*                                                                                           *
//*********************************************************************************************

//
// System Includes
//

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <time.h>

//
// MQ Includes
//

#include <cmqc.h>

//
// Windows Includes
//

#include <windows.h>

//
// Definitions of datatypes
//

#define FULLWORD     long
#define HALFWORD     int
#define MAXSFULLWORD (68719476735)
#define MAXUFULLWORD (4294967295)
#define MAXSHALFWORD (4095)
#define MAXUHALFWORD (65535)
#define MAXFILENAME  (255)

//
// The parms class contains the unravelling of the
//     supplied parameters
//

class parms
{
   private:

       FULLWORD    verbose                          ; // V flag 0=- 1=+
       FULLWORD    syncint                          ; // S flag -1=outside 0=end n=interval
       FULLWORD    persist                          ; // P flag 0=default 1=Pers 2=NotP
       FULLWORD    waiter                           ; // W flag 0=- 1=+
       char        sqm ??(MQ_Q_MGR_NAME_LENGTH+1??) ; // QM    name - source
       char        sq  ??(MQ_Q_NAME_LENGTH+1    ??) ; // Q     name - source
       char        dqm ??(MQ_Q_MGR_NAME_LENGTH+1??) ; // QM    name - destination
       char        dq  ??(MQ_Q_NAME_LENGTH+1    ??) ; // Q     name - destination

       FULLWORD parms::getnumb( char * const ptr ) ;    //Private func to get nums out of parms

   public:
                 parms       () { verbose        =  1        ; //Constructor
                                  syncint        =  0        ; // defaults
                                  persist        =  0        ; //  defaults
                                  waiter         =  1        ; //
                                  memset(sqm,0,sizeof(sqm))  ; //
                                  memset(sq ,0,sizeof(sq ))  ; //
                                  memset(dqm,0,sizeof(dqm))  ; //
                                  memset(dq ,0,sizeof(dq ))  ; //
                                }

       FULLWORD  isverbose   () { if ( verbose == 1 ) return(TRUE)  ;   //Access func to the pa
                                       else           return(FALSE) ; } ;

       FULLWORD  getsyncint  () { return (syncint) ; } ;

       FULLWORD  getpersist  () { return (persist) ; } ;

       FULLWORD  getwaiter   () { return (waiter)  ; } ;

       char    * getsqmname   (char * const dest)
                              {
                                strcpy(dest,sqm) ;
                                return(dest)     ;
                              }

       char    * getsqname    (char * const dest)
                              {
                                strcpy(dest,sq)  ;
                                return(dest)     ;
                              }

       char    * getdqmname   (char * const dest)
                              {
                                strcpy(dest,dqm) ;
                                return(dest)     ;
                              }

       char    * getdqname    (char * const dest)
                              {
                                strcpy(dest,dq)  ;
                                return(dest)     ;
                              }


       void     parmset ( int     const argc,                       //Function to set
                          char ** const argv,                       // up parms in the
                          char ** const envp  ) ;                   // class

} ;

//
// The parmset function is the part of the parms class
//     that builds the parameters
//
// Valid parms are :
//
//            /Maaaaa     -> The Queue Manager Name
//            /Qaaaaa     -> The Queue Name
//            /Faaaaa     -> The File  Name
//            /S - 0 n    -> The Syncpoint Interval (Outside,end,int)
//            /P d + -    -> The Required Message Persistance  (Q,Y,N)
//            /V + -      -> Message Displaying (Y,N)
//            /W + -      -> Wait for input messages (Y,N)
//
void parms::parmset ( int     const argc,
                      char ** const argv,
                      char ** const envp  )
 {
    int     i = 0   ;
    char  * p = 0   ;
    char    c = ' ' ;
    char    s = ' ' ;
    char    v = ' ' ;

    for ( i=1 ; i < argc ; i++ )     // loop through all the parms
        {
          p = argv??(i??) ;          // start of current parm
          c = *p          ;          // should be the selector, so ignore it
          s = *(++p)      ;          // the parm key
          p++             ;          // start of parm data
          v = *p          ;          // first of the parm data

          switch ( s )
           {
             case    'M'  :      // Queue Manager - source
             case    'm'  :

                             strncpy(this->sqm,p,MQ_Q_MGR_NAME_LENGTH) ;
                             break                                     ;

             case    'N'  :      // Queue Manager - destination
             case    'n'  :

                             strncpy(this->dqm,p,MQ_Q_MGR_NAME_LENGTH) ;
                             break                                     ;

             case    'Q'  :      // Queue - source
             case    'q'  :

                             strncpy(this->sq,p,MQ_Q_NAME_LENGTH) ;
                             break                                ;

             case    'R'  :      // Queue - destination
             case    'r'  :

                             strncpy(this->dq,p,MQ_Q_NAME_LENGTH) ;
                             break                                ;


             case    'p'  :      // Persistance required
             case    'P'  :

                             switch ( v )
                                {
                                  case 'd'  :     //Take from Q
                                  case 'D'  :
                                              persist = 0 ;
                                              break       ;

                                  case '+'  :     //Force Persistance
                                              persist = 1 ;
                                              break       ;

                                  case '-'  :     //Force NotPersistance
                                              persist = 2 ;
                                              break       ;

                                  default   : break       ;
                                }
                             break ;

             case    's'  :      // Syncpoint Interval
             case    'S'  :

                             switch ( v )
                                {
                                  case '-'  :     //Outside UOW
                                              syncint = -1         ;
                                              break                ;

                                  default   :     //End of every n
                                              syncint = getnumb(p) ;
                                              break                ;
                                }
                             break ;

             case    'v'  :      // Message usage
             case    'V'  :

                             switch ( v )
                                {

                                  case '+'  :     //Yes
                                              verbose = 1 ;
                                              break       ;

                                  case '-'  :     //No
                                              verbose = 0 ;
                                              break       ;

                                  default   : break       ;
                                }
                             break ;

             case    'w'  :      // Message waiting
             case    'W'  :

                             switch ( v )
                                {

                                  case '+'  :     //Yes
                                              waiter = 1 ;
                                              break      ;

                                  case '-'  :     //No
                                              waiter = 0 ;
                                              break      ;

                                  default   : break      ;
                                }
                             break ;


             default :
                        break  ;       // unknown parm, so ignore it
           } ;

        } ;

    return ;

 }

//
// The getnumb private function within the parms class
//     merely extracts the numeric part of a /Pnnnn
//
FULLWORD parms::getnumb( char * const ptr )
 {
    FULLWORD   numb = 0   ;
    char     * p    = 0   ;
    char       c    = ' ' ;

    for ( p = ptr ; *p != '\0' ; p++ )
        {
          c = *p ;
          if ( c == ' ' ) break ;
          if ( ( c >= '0' ) && ( c <= '9' ) )
                numb = ( numb * 10 ) + ( c - '0' ) ;
        }
    return ( numb ) ;
  }



//
// RAHMQN Class Library
//

class rahmqn
 {
   public :

    char        qm  ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name
    char        q   ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name
    MQHCONN     qmh                              ; //QM handle
    MQHOBJ      qh                               ; //Q  handle
    MQLONG      rc                               ; //Return Code
    MQLONG      ac                               ; //Reason Code
    MQLONG      lastrc                           ; //Prev Return Code
    MQLONG      lastac                           ; //Prev Reason Code


    rahmqn ( void )               ;  //Contructor for new  use of QM
    rahmqn ( MQHCONN const uqmh)  ;  //Contructor for same use of QM

    MQLONG      conn ( void )               ;  //Connect    to QM
    MQLONG      conn ( char * const uqm )   ;  //  with QM name supplied

    MQLONG      disc ( void )               ;  //Disconnect from QM

    void        cleanup ( void )            ;  //Close and Disconnect

    MQOD        odopen                      ;  //OD for MQOPEN usage
    MQLONG      open (MQLONG const uopts )  ;  //MQOPEN call
    MQLONG      open (char * const uq    ,     //  with Q name supplied
                      MQLONG const uopts )  ;  //
    MQLONG      open (MQOD * const uod   ,     //  with OD and options
                      MQLONG const uopts )  ;  //

    MQLONG      close (MQLONG const uopts ) ;  //MQCLOSE call

    MQLONG      commit   ( void ) ; //Syncpoint UOW
    MQLONG      rollback ( void ) ; //Backout   UOW

    MQLONG      inq ( MQLONG   const  uselnum ,             //MQINQ call
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  ) ;

    MQLONG      set ( MQLONG   const uselnum  ,             //MQSET call
                      MQLONG         usels    ,
                      MQLONG   const uinum    ,
                      MQLONG *       uiattr   ,
                      MQLONG   const ucharl   ,
                      char   *       ucattr   ) ;

    MQMD        pmd                             ;           //PUT Message Dwescriptor
    MQLONG      put ( MQPMO  * const  uopts   ,             //MQPUT call
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

    MQLONG      put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  ) ;

                                                            //PUT Zero-Length record
    MQLONG      putz ( MQPMO  * const  uopts     ) ;        //No Tag, Use Instance MD

    MQLONG      putz ( MQMD   *        upmd      ,          //No Tag, use supplied MD
                       MQPMO  * const  uopts     ) ;

    MQLONG      putz ( MQPMO    * const  uopts   ,          //Tag, Use Instance MD
                       MQBYTE24 * const  utag    ) ;

    MQLONG      putz ( MQMD     *        upmd    ,          //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    ) ;

    MQMD        bmd                                  ;      //Browse Message Descriptor
    MQGMO       bmo                                  ;      //Browse MQGET options
    MQLONG      browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;

    MQLONG      browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  ) ;


    MQMD        gmd                               ;         //GET Message Descriptor
    MQLONG      get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;

    MQLONG      get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  ) ;


    double      gettimeconn     () { return (time_conn     ) ; }
    double      gettimedisc     () { return (time_disc     ) ; }
    double      gettimeopen     () { return (time_open     ) ; }
    double      gettimeclose    () { return (time_close    ) ; }
    double      gettimeinq      () { return (time_inq      ) ; }
    double      gettimeset      () { return (time_set      ) ; }
    double      gettimeput      () { return (time_put      ) ; }
    double      gettimeget      () { return (time_get      ) ; }
    double      gettimecommit   () { return (time_commit   ) ; }
    double      gettimerollback () { return (time_rollback ) ; }
    double      getcputconn     () { return (cput_conn     / CLOCKS_PER_SEC) ; }
    double      getcputdisc     () { return (cput_disc     / CLOCKS_PER_SEC) ; }
    double      getcputopen     () { return (cput_open     / CLOCKS_PER_SEC) ; }
    double      getcputclose    () { return (cput_close    / CLOCKS_PER_SEC) ; }
    double      getcputinq      () { return (cput_inq      / CLOCKS_PER_SEC) ; }
    double      getcputset      () { return (cput_set      / CLOCKS_PER_SEC) ; }
    double      getcputput      () { return (cput_put      / CLOCKS_PER_SEC) ; }
    double      getcputget      () { return (cput_get      / CLOCKS_PER_SEC) ; }
    double      getcputcommit   () { return (cput_commit   / CLOCKS_PER_SEC) ; }
    double      getcputrollback () { return (cput_rollback / CLOCKS_PER_SEC) ; }

 private :

    double      time_conn      ;
    double      time_disc      ;
    double      time_open      ;
    double      time_close     ;
    double      time_inq       ;
    double      time_set       ;
    double      time_put       ;
    double      time_get       ;
    double      time_commit    ;
    double      time_rollback  ;


    double      cput_conn      ;
    double      cput_disc      ;
    double      cput_open      ;
    double      cput_close     ;
    double      cput_inq       ;
    double      cput_set       ;
    double      cput_put       ;
    double      cput_get       ;
    double      cput_commit    ;
    double      cput_rollback  ;
 } ;

//
// rahmqn member function definitions
//

//
// Constructor functions
//
//     without a parm, then this is a new use of a QM
//     with          , then this is an additional access on a
//                     previously connected QM
//

rahmqn::rahmqn ( void )
{
  memset(qm,0,sizeof(qm)) ;
  memset(q ,0,sizeof(q )) ;
  qmh    = 0              ;
  qh     = 0              ;
  rc     = 0              ;
  ac     = 0              ;
  lastrc = 0              ;
  lastac = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

rahmqn::rahmqn ( MQHCONN const uqmh )
{
  memset(qm,0,sizeof(qm)) ;
  memset(q ,0,sizeof(q )) ;
  qmh    = uqmh           ;
  qh     = 0              ;
  rc     = 0              ;
  ac     = 0              ;
  lastrc = 0              ;
  lastac = 0              ;
  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  memset(&pmd,0,sizeof(pmd))                                   ;
  memset(&bmd,0,sizeof(bmd))                                   ;
  memset(&gmd,0,sizeof(gmd))                                   ;

  cput_conn      = 0 ;
  cput_disc      = 0 ;
  cput_open      = 0 ;
  cput_close     = 0 ;
  cput_inq       = 0 ;
  cput_set       = 0 ;
  cput_put       = 0 ;
  cput_get       = 0 ;
  cput_commit    = 0 ;
  cput_rollback  = 0 ;
  time_conn      = 0 ;
  time_disc      = 0 ;
  time_open      = 0 ;
  time_close     = 0 ;
  time_inq       = 0 ;
  time_set       = 0 ;
  time_put       = 0 ;
  time_get       = 0 ;
  time_commit    = 0 ;
  time_rollback  = 0 ;

  return             ;
}

//
// rahmqn destructor functions
//
//   these are not provided
//
//    (Note that a disc call removes access to ALL
//     instances of RAHMQN sharing the Handle. as I
//     am not aiming at a 'proper' C++ implementation
//     I have not provided code to cope with this event).
//

//
// rahmqn functions to access a Queue Manager
//

//
// conn  will connect to a Queue Manager
//       saving the QMH
//
//       The version with no parms will use the
//           pre-set Queue Manager name
//
//       The alternative access is to quote a
//           Queue Manager Name which is the
//           one to be used
//

MQLONG  rahmqn::conn ( char * const uqm )
{
  double c1, c2, t1, t2         ;
  strncpy(qm,uqm,sizeof(qm)-1)  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::conn ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCONN ( qm, &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_conn += 1.0000 * ( c2 - c1 )      ;
  time_conn += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// disc  will disconnect from a Queue Manager
//       deleting the QMH
//


MQLONG  rahmqn::disc ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQDISC ( &qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_disc += 1.0000 * ( c2 - c1 )      ;
  time_disc += 1.0000 * difftime(t2,t1)  ;

  qmh = 0     ;

  return (ac) ;
}

//
// cleanup will Close the Q (if known) and Disconnect from
//         the QM (if known). This will effect other instances
//         which were created with a QM handle
//

void    rahmqn::cleanup ( void )
{
  double c1, c2, t1, t2         ;

  if ( qh  != 0 ) close(MQCO_NONE) ;
  if ( qmh != 0 ) disc ()          ;
  return                           ;
}

//
// commit will Syncpoint ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqn::commit ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCMIT ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_commit += 1.0000 * ( c2 - c1 )      ;
  time_commit += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// rollback will backout ALL accesses to the
//       QM on the given QMH (this will effect
//       other instances of the Class if the
//       QMH is being shared) in the current UOW
//

MQLONG  rahmqn::rollback ( void )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQBACK ( qmh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_rollback += 1.0000 * ( c2 - c1 )      ;
  time_rollback += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// open  will provide access to a Queue
//
//       The OPEN options are always supplied
//           by the caller.
//
//       If the version without a Queue Name is
//          used, then the Name from the Instance
//          is taken.
//

MQLONG  rahmqn::open ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqn::open ( char * const uq    ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  strncpy(q,uq,sizeof(q)-1)   ;

  memset(&odopen,0,sizeof(odopen))                             ;
  strncpy(odopen.StrucId,MQOD_STRUC_ID,sizeof(odopen.StrucId)) ;
  odopen.Version    = MQOD_VERSION_1                           ;
  odopen.ObjectType = MQOT_Q                                   ;
  strcpy(odopen.ObjectName,q)                                  ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqn::open ( MQOD * const uod   ,
                       MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  memcpy(&odopen,uod,sizeof(odopen)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQOPEN ( qmh, &odopen, uopts, &qh, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_open += 1.0000 * ( c2 - c1 )      ;
  time_open += 1.0000 * difftime(t2,t1)  ;

  memcpy(uod,&odopen,sizeof(odopen))     ;
  return (ac) ;
}

//
// close will stop access to a Queue
//
//       The CLOSE options are always supplied
//           by the caller.
//

MQLONG  rahmqn::close ( MQLONG const uopts )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQCLOSE ( qmh, &qh, uopts, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_close += 1.0000 * ( c2 - c1 )      ;
  time_close += 1.0000 * difftime(t2,t1)  ;

  qh = 0      ;

  return (ac) ;
}

//
// inq   will inquire upon the Attributes of
//       a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqn::inq ( MQLONG   const  uselnum ,
                      MQLONG          usels   ,
                      MQLONG   const  uinum   ,
                      MQLONG *        uiattr  ,
                      MQLONG   const  ucharl  ,
                      char   *        ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQINQ ( qmh, qh, uselnum, &usels ,
                   uinum  , uiattr ,
                   ucharl , ucattr ,
                                     &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_inq += 1.0000 * ( c2 - c1 )      ;
  time_inq += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}
//
// set   will set the Attributes of a Queue etc.
//
//       Note that ALL the Selectors stuff
//       has to be supplied by the caller
//

MQLONG  rahmqn::set ( MQLONG   const   uselnum ,
                      MQLONG           usels   ,
                      MQLONG   const   uinum   ,
                      MQLONG *         uiattr  ,
                      MQLONG   const   ucharl  ,
                      char   *         ucattr  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQSET ( qmh, qh, uselnum , &usels  ,
                   uinum   , uiattr  ,
                   ucharl  , ucattr  ,
                                       &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_set += 1.0000 * ( c2 - c1 )      ;
  time_set += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


//
// put       will write a Message into a Queue
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqn::put ( MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::put ( MQMD   *        upmd    ,
                      MQPMO  * const  uopts   ,
                      MQLONG   const  bufferl ,
                      char   * const  buffer  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, bufferl, buffer, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// putz      will write a Zero-Length Message into a Queue
//
//           If a Tag is supplied, then this is used as the Message Descriptor
//              this must be an area of MQBYTE24 (24 bytes long)
//
//           The Message descriptor may either
//           be supplied or the instance version
//           used.
//

MQLONG  rahmqn::putz ( MQPMO  * const  uopts )               //No Tag, Use Instance MD
{
  double c1, c2, t1, t2         ;

  memset(&this->pmd.MsgId,0,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::putz ( MQMD   *        upmd    ,              //No Tag, use supplied MD
                       MQPMO  * const  uopts   )
{
  double c1, c2, t1, t2         ;

  memset(upmd->MsgId,0,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

MQLONG  rahmqn::putz ( MQPMO    * const  uopts   ,            //Tag, Use Instance MD
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(&this->pmd.MsgId,utag,sizeof(this->pmd.MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, &pmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::putz ( MQMD     *        upmd    ,            //Tag, use supplied MD
                       MQPMO    * const  uopts   ,
                       MQBYTE24 * const  utag    )
{
  double c1, c2, t1, t2         ;

  memcpy(upmd->MsgId,utag,sizeof(upmd->MsgId)) ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQPUT ( qmh, qh, upmd, uopts, 0, 0, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_put += 1.0000 * ( c2 - c1 )      ;
  time_put += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
//    Browse    will non-destructively read a Queue
//
//              Versions are supplied that use an
//              instance Message dewscriptor, or
//              one supplied by the caller
//

MQLONG  rahmqn::browse ( MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_FAIL_IF_QUIESCING               ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  memset(&bmd,0,sizeof(bmd))                                 ;
  memcpy(&bmd.StrucId,MQMD_STRUC_ID,sizeof(bmd.StrucId))     ;
  bmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &bmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::browse ( MQMD   *        ubmd      ,
                         MQLONG   const  bufferl   ,
                         char   * const  buffer    ,
                         MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&bmo,0,sizeof(bmo))                                 ;
  memcpy(&bmo.StrucId,MQGMO_STRUC_ID,sizeof(bmo.StrucId))    ;
  bmo.Version        = MQGMO_VERSION_1                       ;
  bmo.Options        = MQGMO_NO_WAIT+MQGMO_BROWSE_NEXT+MQGMO_ACCEPT_TRUNCATED_MSG+
                       MQGMO_CONVERT+MQGMO_FAIL_IF_QUIESCING ;
  bmo.WaitInterval   = MQWI_UNLIMITED                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ubmd, &bmo, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::get ( MQGMO  * const  uopts     ,           //MQGET call
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  bmo.Version        = MQMD_VERSION_1                        ;

  memset(&gmd,0,sizeof(gmd))                                 ;
  memcpy(&gmd.StrucId,MQMD_STRUC_ID,sizeof(gmd.StrucId))     ;
  gmd.Version        = MQMD_VERSION_1                        ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, &gmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}


MQLONG  rahmqn::get ( MQMD   *        ugmd      ,
                      MQGMO  * const  uopts     ,
                      MQLONG   const  bufferl   ,
                      char   * const  buffer    ,
                      MQLONG *        udatalen  )
{
  double c1, c2, t1, t2         ;

  lastrc = rc ;
  lastac = ac ;

  c1 = clock()    ;
  t1 = time(NULL) ;
  MQGET ( qmh, qh, ugmd, uopts, bufferl, buffer, udatalen, &rc, &ac ) ;
  c2 = clock()    ;
  t2 = time(NULL) ;

  cput_get += 1.0000 * ( c2 - c1 )      ;
  time_get += 1.0000 * difftime(t2,t1)  ;

  return (ac) ;
}

//
// End of RAHMQN Class definition and functions
//


//
// Main procedure - Move a Queue to a Queue
//
int  main ( int     const argc,
            char ** const argv,
            char ** const envp  )
{
 parms       myparms        ;
 rahmqn      smq            ; //MQ instances - source
 rahmqn      dmq            ; //             - destination

 char        sqm  ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name - source
 char        sq   ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name
 char        dqm  ??(MQ_Q_MGR_NAME_LENGTH+1??) ; //QM name - destination
 char        dq   ??(MQ_Q_NAME_LENGTH+1    ??) ; //Q  name

 FULLWORD    syncint  = 0    ;
 FULLWORD    persist  = 0    ;
 FULLWORD    waiter   = 0    ;
 MQLONG      gopts    = 0    ;
 MQLONG      popts    = 0    ;
 MQGMO       gmo             ;
 MQPMO       pmo             ;

 MQLONG      smaxmsgl = 0    ;
 MQLONG      dmaxmsgl = 0    ;
 char     *  buffer   = 0    ;
 FULLWORD    maxnum   = 0    ;

 double      totcput  = 0    ;
 double      tottime  = 0    ;
 double      totc1    = 0    ;
 double      totc2    = 0    ;
 double      tote1    = 0    ;
 double      tote2    = 0    ;

 MQLONG      mqrc     = 0    ;
 MQLONG      mqac     = 0    ;
 MQLONG      ac       = 0    ;
 MQLONG      rc       = 0    ;

 FULLWORD    syncno   = 0    ;
 FULLWORD    totsync  = 0    ;
 FULLWORD    recno   =  0    ;

 totc1 = clock()    ;                      //Set Timers
 tote1 = time(NULL) ;

 myparms.parmset(argc, argv, envp)      ;  //set the parms

 myparms.getsqmname(sqm)          ;    //Get the data
 myparms.getsqname (sq )          ;    // from the
 myparms.getdqmname(dqm)          ;    // parms
 myparms.getdqname (dq )          ;
 syncint = myparms.getsyncint()   ;
 persist = myparms.getpersist()   ;
 waiter  = myparms.getwaiter ()   ;

 if ( myparms.isverbose() )          //Display program parms
    {
      cout << "Program parms are: " ;
      for ( int i=0 ; i<argc ; cout << argv??(i++??) << " " ) ;
      cout << endl ;
    }

 if ( argc == 1 )                    //Show parms ONLY if none given
    {
      cout << "\nMQNQMTOQ will move a Queue to another Queue \n\n"
           << "Parms for MQNQMTOQ are /M<source queue manager>  /Q<source queue> \n"
           << "                       /N<dest   queue manager>  /R<dest   queue> \n"
           << "                       /P<persistance>           /V<verbose> \n"
           << "                       /S<sync interval>         /W<wait> \n\n"
           << " /Pd from Qdef         /P+ Force Persistance   /P- Force Non-Persistance \n"
           << " /S- Run outside UOW   /S0 Sync at end         /Sn Sync every n \n"
           << " /V+ display           /V- no messages \n"
           << " /W+ wait for inp mess /W- don't wait \n\n"
           ;
      return (0) ;
    }

  if ( myparms.isverbose() ) switch ( syncint )        //Display UoW info
   {
     case -1 :
              cout << "Messages to be read Outside of a UoW" << endl ;
              break                                                  ;

     case 0  :
              cout << "Messages(within a UoW) are Syncpointed after all have been read from the Q" << endl ;
              break                                                                                        ;

     default :
              cout << "Messages(within a UoW) are Syncpointed every " << syncint << " messages" << endl ;
              break                                                                                     ;
   }

  if ( myparms.isverbose() ) switch ( waiter )         //Display Wait info
   {
     case 0  :
              cout << "Messages will not be waited for" << endl ;
              break                                                  ;

     default :
              cout << "Messages will be waited for" << endl ;
              break                                                                                     ;
   }

 switch ( syncint )              //Set Get/Put options
  {
    case -1 :
               gopts  = MQGMO_NO_SYNCPOINT         +
                        ((waiter == 0) ? MQGMO_NO_WAIT : MQGMO_WAIT ) +
                        MQGMO_ACCEPT_TRUNCATED_MSG +
                        MQGMO_FAIL_IF_QUIESCING    ;
               popts  = MQPMO_NO_SYNCPOINT         +
                        MQPMO_NO_CONTEXT           +
                        MQPMO_FAIL_IF_QUIESCING    ;
               break                               ;

    default :
               gopts  = MQGMO_SYNCPOINT            +
                        ((waiter == 0) ? MQGMO_NO_WAIT : MQGMO_WAIT ) +
                        MQGMO_ACCEPT_TRUNCATED_MSG +
                        MQGMO_FAIL_IF_QUIESCING    ;
               popts  = MQPMO_SYNCPOINT            +
                        MQPMO_NO_CONTEXT           +
                        MQPMO_FAIL_IF_QUIESCING    ;
               break                               ;
  }


 if ( rc == 0 )                      // Connect to QM - source
    {
      if ( myparms.isverbose() ) cout << "Connecting to source QM " << sqm << " " ;
      ac   = smq.conn(sqm)   ;
      mqrc = smq.lastrc      ;
      mqac = smq.lastrc      ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Connect to QM - destination
    {
      if ( strcmp(sqm,dqm) != 0 )
         {
           if ( myparms.isverbose() ) cout << "Connecting to destination QM " << dqm << " " ;
           ac   = dmq.conn(dqm)   ;
           mqrc = smq.lastrc      ;
           mqac = smq.lastrc      ;

           if ( ac != 0 ) rc = ac ;
           if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
          }
          else
          {
            cout << "Reusing Connection Handle for destination QM" << endl ;
            dmq.qmh = smq.qmh ;
          }
    }

 if ( rc == 0 )                      // Open the source Q for INQ/Get access
    {
      if ( myparms.isverbose() ) cout << "Opening Q " << sq << " for Destructive access " ;
      ac   = smq.open(sq,(MQOO_INQUIRE+MQOO_INPUT_EXCLUSIVE
                                      +MQOO_BROWSE+MQOO_SAVE_ALL_CONTEXT
                                      +MQOO_FAIL_IF_QUIESCING))                           ;
      mqrc = smq.lastrc     ;
      mqac = smq.lastrc     ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Open the destination Q for INQ/Put access
    {
      if ( myparms.isverbose() ) cout << "Opening Q " << dq << " for Output " ;
      ac   = dmq.open(dq,(MQOO_INQUIRE+MQOO_OUTPUT+MQOO_FAIL_IF_QUIESCING))   ;
      mqrc = dmq.lastrc     ;
      mqac = dmq.lastrc     ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Get source MaxMessage Size
    {
      MQLONG   tselcount        = 1                   ;
      MQLONG   tsels            = MQIA_MAX_MSG_LENGTH ;
      MQLONG   tintcount        = 1                   ;
      MQLONG   tintattr         = 0                   ;
      MQLONG   tcharlen         = 0                   ;
      char     tcharattr        = '\0'                ;

      if ( myparms.isverbose() ) cout << "Obtaining source Q Max Message Length " ;
      ac   =  smq.inq(tselcount, tsels,
                                        tintcount, &tintattr  ,
                                        tcharlen , &tcharattr ) ;

      mqrc = smq.lastrc           ;
      mqac = smq.lastrc           ;

      smaxmsgl = tintattr         ;

      if ( ac != 0 ) rc = ac      ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << " Max Len is " << smaxmsgl << endl ;

      if ( smaxmsgl > MAXUHALFWORD )
         {
           smaxmsgl = MAXUHALFWORD ;
           if ( myparms.isverbose() ) cout << "which is forced to max supported size of " << MAXUHALFWORD << endl ;
         }
    }

 if ( rc == 0 )                      // Get destination MaxMessage Size
    {
      MQLONG   tselcount        = 1                   ;
      MQLONG   tsels            = MQIA_MAX_MSG_LENGTH ;
      MQLONG   tintcount        = 1                   ;
      MQLONG   tintattr         = 0                   ;
      MQLONG   tcharlen         = 0                   ;
      char     tcharattr        = '\0'                ;

      if ( myparms.isverbose() ) cout << "Obtaining destination Q Max Message Length " ;
      ac   =  dmq.inq(tselcount, tsels,
                                        tintcount, &tintattr  ,
                                        tcharlen , &tcharattr ) ;

      mqrc = dmq.lastrc           ;
      mqac = dmq.lastrc           ;

      dmaxmsgl = tintattr         ;

      if ( ac != 0 ) rc = ac      ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << " Max Len is " << dmaxmsgl << endl ;

      if ( dmaxmsgl > MAXUHALFWORD )
         {
           dmaxmsgl = MAXUHALFWORD ;
           if ( myparms.isverbose() ) cout << "which is forced to max supported size of " << MAXUHALFWORD << endl ;
         }
    }

 if ( rc == 0 )  //Ensure all source messages will fit in the destination Q
    {
      if ( smaxmsgl > dmaxmsgl )
         {
           if ( myparms.isverbose() ) cout << "Source Max Message Length of "    << smaxmsgl
                                           << " greater than destination\'s of " << dmaxmsgl << endl ;
           rc = -7 ;
         }
    }

 if ( rc == 0 )                      // Get Number of Messages in the  source Q
    {
      MQLONG   tselcount        = 1                    ;
      MQLONG   tsels            = MQIA_CURRENT_Q_DEPTH ;
      MQLONG   tintcount        = 1                    ;
      MQLONG   tintattr         = 0                    ;
      MQLONG   tcharlen         = 0                    ;
      char     tcharattr        = '\0'                 ;

      if ( myparms.isverbose() ) cout << "Obtaining current source Q depth " ;
      ac   =  smq.inq(tselcount, tsels,
                                        tintcount, &tintattr  ,
                                        tcharlen , &tcharattr ) ;

      mqrc = smq.lastrc          ;
      mqac = smq.lastrc          ;

      maxnum  = tintattr         ;

      if ( ac != 0 ) rc = ac     ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << " Depth is " << maxnum << endl ;

    }

 if ( rc == 0 )                      // Getmain the Buffer (always max file size)
    {
      if ( myparms.isverbose() ) cout << "Getmaining the File/Q Buffer " ;
      buffer = (char *)malloc(MAXUHALFWORD) ;
      if ( buffer == 0 ) rc = -8    ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }


 if ( rc == 0 )                      // Read Q and Write Q
    {
      for ( recno = 0 , syncno = 1 ; recno < maxnum ; recno++, syncno++ )
          {
            MQLONG   i      = 0   ;
            char     exiter = 'N' ;
            char   * p      = 0   ;
            MQLONG   mrc    = 0   ;
            MQLONG   frc    = 0   ;
            FULLWORD reclen = 0   ;

            memset(&gmo,0,sizeof(gmo))                                      ;
            memcpy(&gmo.StrucId,MQGMO_STRUC_ID,sizeof(gmo.StrucId))         ;
            gmo.Version        = MQGMO_VERSION_1                            ;
            gmo.Options        = gopts                                      ;
            gmo.WaitInterval   = MQWI_UNLIMITED                             ;

            memset(&smq.gmd,0,sizeof(smq.gmd))                              ;
            memcpy(&smq.gmd.StrucId,MQMD_STRUC_ID,sizeof(smq.gmd.StrucId))  ;
            smq.gmd.Version        = MQMD_VERSION_1                         ;

            memset(buffer,0,MAXUHALFWORD)                                   ;
            reclen   = 0                                                    ;
            mrc = smq.get ( &gmo, MAXUHALFWORD, buffer, &reclen )           ;

            switch ( mrc )
             {
               case MQRC_NONE                   :
                    rc   = 0      ;
                    break         ;

               default                          :
                    if ( myparms.isverbose() ) cout << "MQ GET error RC=" << mrc
                                                    <<  " on message "    << recno+1 << endl ;
                    exiter = 'G'  ;
                    rc  = mrc     ;
                    break         ;

             }

            memset(&pmo,0,sizeof(pmo))  ;               //Set Put Message Options
            memcpy(&pmo.StrucId,MQPMO_STRUC_ID,sizeof(pmo.StrucId))        ;
            pmo.Version        = MQPMO_VERSION_1                           ;
            pmo.Options        = popts                                     ;

            memset(&dmq.pmd,0,sizeof(dmq.pmd))  ;     //Set Put Message Desc
            memcpy(&dmq.pmd.StrucId,MQMD_STRUC_ID,sizeof(dmq.pmd.StrucId)) ;
            memcpy(&dmq.pmd.Format ,MQFMT_NONE   ,sizeof(dmq.pmd.Format )) ;
            dmq.pmd.Version        = MQMD_VERSION_1                        ;
            dmq.pmd.Report         = MQRO_NONE                             ;
            dmq.pmd.MsgType        = MQMT_DATAGRAM                         ;
            dmq.pmd.Expiry         = MQEI_UNLIMITED                        ;
            dmq.pmd.Feedback       = MQFB_NONE                             ;
            dmq.pmd.Encoding       = MQENC_NATIVE                          ;
            dmq.pmd.CodedCharSetId = MQCCSI_Q_MGR                          ;
            dmq.pmd.Priority       = MQPRI_PRIORITY_AS_Q_DEF               ;
            dmq.pmd.PutApplType    = MQAT_OS2                              ;
            switch ( persist )
             {
               case 0 :
                        dmq.pmd.Persistence = MQPER_PERSISTENCE_AS_Q_DEF  ;
                        break                                             ;

               case 1 :
                        dmq.pmd.Persistence = MQPER_PERSISTENT            ;
                        break                                             ;

               case 2 :
                        dmq.pmd.Persistence = MQPER_NOT_PERSISTENT        ;
                        break                                             ;

               default:
                        break                                             ;
             }

            if ( reclen != 0 ) frc = dmq.put  ( &pmo, reclen, buffer )    ;
               else            frc = dmq.putz ( &pmo                 )    ;

            switch ( mrc )
             {
               case MQRC_NONE                   :
                    rc   = 0      ;
                    break         ;

               default                          :
                    if ( myparms.isverbose() ) cout << "MQ PUT error RC=" << mrc
                                                    <<  " on message "    << recno+1 << endl ;
                    exiter = 'P'  ;
                    rc  = mrc     ;
                    break         ;

             }

            if ( ( rc == 0 ) && ( reclen == 0 ) ) exiter = 'Y' ;   //Zero-Length record = EOQ

                                                //Syncpoint if its
                                                //to be done after
                                                //each GET/PUT, or if
                                                //the required number
                                                //of GET/PUTs have been
                                                //done (or EOQ hit)

            if ( ( rc == 0 ) && ( syncint >= 0 ) )
               {
                 if (    ( ( syncint >  0       ) && ( syncno  >= syncint ) )
                      || (   exiter  == 'Y'     )  )
                    {
                      if ( ( exiter == 'Y' ) && ( myparms.isverbose() )  )
                          cout << "Initiating final Syncpoint " ;

                      syncno = 0         ;
                      totsync++          ;

                      rc = dmq.commit()  ; //which commits smq as well (same UoW)

                      if ( ( exiter == 'Y' ) && ( myparms.isverbose() )  )
                          cout << "RC=" << rc << endl ;

                      if ( rc != 0 )
                         {
                           if ( myparms.isverbose() ) cout << "Syncpoint Error RC=" << rc << endl ;
                         }
                    }
               } // End of Syncpoint Block

                                                //If there was a
                                                //failure of any sort
                                                //whilsr running
                                                //inside a Uow, then
                                                //Backout any updates

              if ( ( rc != 0 ) && ( syncint >= 0 ) )
                 {
                   exiter = 'X'          ;
                   ac = dmq.rollback()   ; //which backsout smq as well (same UoW)
                   if ( ac != 0 )
                      {
                        if ( myparms.isverbose() ) cout << "Rollback Error RC=" << ac << endl ;
                      }
                 } // End of Rollback Block

              if ( ( rc == 0 ) && ( recno >= maxnum ) ) exiter = 'Y' ; //Exit due to all messages read OK

              if ( exiter != 'N' ) break ; //Get next message
                                           //unless error/EOQ occured

          } // End of Message Processing FOR loop

    } // End of Queue/Queue Processing Block

if ( ( rc == 0 ) && ( myparms.isverbose() ) ) cout << "All Q records moved" << endl ;

                                                //End of processing

 if ( rc == 0 )                      // Close the Q - destination
    {
      if ( myparms.isverbose() ) cout << "Closing the destination Q " ;
      ac   = dmq.close(MQCO_NONE)  ;
      mqrc = dmq.lastrc            ;
      mqac = dmq.lastrc            ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Close the Q - source
    {
      if ( myparms.isverbose() ) cout << "Closing the source Q " ;
      ac   = smq.close(MQCO_NONE)  ;
      mqrc = smq.lastrc            ;
      mqac = smq.lastrc            ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }

 if ( rc == 0 )                      // Disconnect from destination QM
    {
      if ( strcmp(sqm,dqm) != 0 )
         {
           if ( myparms.isverbose() ) cout << "Disconnecting from destination QM " ;
           ac   = dmq.disc()            ;
           mqrc = dmq.lastrc            ;
           mqac = dmq.lastrc            ;

           if ( ac != 0 ) rc = ac ;
           if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
         }
         else
         {
           cout << "Dropping duplicate destination QM handle" << endl ;
           dmq.qmh = 0 ;
         }
    }

 if ( rc == 0 )                      // Disconnect from source QM
    {
      if ( myparms.isverbose() ) cout << "Disconnecting from source QM " ;
      ac   = smq.disc()            ;
      mqrc = smq.lastrc            ;
      mqac = smq.lastrc            ;

      if ( ac != 0 ) rc = ac ;
      if ( myparms.isverbose() ) cout << "RC=" << rc << endl ;
    }


 if ( ( rc == 0 ) && ( myparms.isverbose() ) )  //Issue Good completion message
    {

      totc2 = clock()    ;
      tote2 = time(NULL) ;
      totcput = 1.0000 * ( totc2 - totc1  ) / CLOCKS_PER_SEC   ;
      tottime = 1.0000 * difftime(tote2,tote1)                 ;

      cout << recno << " records moved from Q " << sq << " to Q " << dq << "\n"            ;
      cout << "CPU time (ms) and Elapsed Time (s) per MQ verb follow:\n"                   ;
      cout << " >Connect    " << smq.getcputconn     () +  dmq.getcputconn     () <<
                          " " << smq.gettimeconn     () +  dmq.gettimeconn     () << "\n"  ;
      cout << " >Disconnect " << smq.getcputdisc     () +  dmq.getcputdisc     () <<
                          " " << smq.gettimedisc     () +  dmq.gettimedisc     () << "\n"  ;
      cout << " >Open       " << smq.getcputopen     () +  dmq.getcputopen     () <<
                          " " << smq.gettimeopen     () +  dmq.gettimeopen     () << "\n"  ;
      cout << " >Close      " << smq.getcputclose    () +  dmq.getcputclose    () <<
                          " " << smq.gettimeclose    () +  dmq.gettimeclose    () << "\n"  ;
      if ( totsync == 0 )
           cout << " >Commit     " << ( smq.getcputcommit   ()  +  dmq.getcputcommit   () )           <<
                              "  " << ( smq.gettimecommit   ()  +  dmq.gettimecommit   () )           << "\n" ;
           else
           cout << " >Commit     " << ( smq.getcputcommit   ()  +  dmq.getcputcommit   () )           <<
                              "  " << ( smq.gettimecommit   ()  +  dmq.gettimecommit   () )           <<
                   " per call    " << ( smq.getcputcommit   ()  +  dmq.getcputcommit   () ) / totsync <<
                              "  " << ( smq.gettimecommit   ()  +  dmq.gettimecommit   () ) / totsync << "\n" ;
      cout << " >Rollback   " << smq.getcputrollback () +  dmq.getcputrollback () <<
                          " " << smq.gettimerollback () +  dmq.gettimerollback () << "\n"  ;
      cout << " >Inquire    " << smq.getcputinq      () +  dmq.getcputinq      () <<
                          " " << smq.gettimeinq      () +  dmq.gettimeinq      () << "\n"  ;
      cout << " >Set        " << smq.getcputset      () +  dmq.getcputset      () <<
                          " " << smq.gettimeset      () +  dmq.gettimeset      () << "\n"  ;
      cout << " >Get        " <<  ( smq.getcputget() + dmq.getcputget() )              <<
                         "  " <<  ( smq.gettimeget() + dmq.gettimeget() )              <<
              " per record  " <<  ( smq.getcputget() + dmq.getcputget() ) / (recno+1)  <<
                         "  " <<  ( smq.gettimeget() + dmq.gettimeget() ) / (recno+1)  << "\n" ;
      cout << " >Put        " <<  ( smq.getcputput() + dmq.getcputput() )              <<
                         "  " <<  ( smq.gettimeput() + dmq.gettimeput() )              <<
              " per record  " <<  ( smq.getcputput() + dmq.getcputput() ) / (recno+1)  <<
                         "  " <<  ( smq.gettimeput() + dmq.gettimeput() ) / (recno+1)  << "\n" ;
      cout << "Total MQ CPU time " << (   smq.getcputconn     ()  +  dmq.getcputconn     ()
                                        + smq.getcputdisc     ()  +  dmq.getcputdisc     ()
                                        + smq.getcputopen     ()  +  dmq.getcputopen     ()
                                        + smq.getcputclose    ()  +  dmq.getcputclose    ()
                                        + smq.getcputcommit   ()  +  dmq.getcputcommit   ()
                                        + smq.getcputrollback ()  +  dmq.getcputrollback ()
                                        + smq.getcputinq      ()  +  dmq.getcputinq      ()
                                        + smq.getcputset      ()  +  dmq.getcputset      ()
                                        + smq.getcputget      ()  +  dmq.getcputget      ()
                                        + smq.getcputput      ()  +  dmq.getcputput      () ) << "s "
           << "Total MQ Elapsed time " << (   smq.gettimeconn     ()   +  dmq.gettimeconn     ()
                                            + smq.gettimedisc     ()   +  dmq.gettimedisc     ()
                                            + smq.gettimeopen     ()   +  dmq.gettimeopen     ()
                                            + smq.gettimeclose    ()   +  dmq.gettimeclose    ()
                                            + smq.gettimecommit   ()   +  dmq.gettimecommit   ()
                                            + smq.gettimerollback ()   +  dmq.gettimerollback ()
                                            + smq.gettimeinq      ()   +  dmq.gettimeinq      ()
                                            + smq.gettimeset      ()   +  dmq.gettimeset      ()
                                            + smq.gettimeget      ()   +  dmq.gettimeget      ()
                                            + smq.gettimeput      ()   +  dmq.gettimeput      () ) << "s\n" ;
      cout << "Total CPU " << totcput << "s, Total elapsed time " << tottime << "s" << endl ;

    }

 if ( buffer != 0 ) free(buffer)      ; //Freemain the buffer
 dmq.cleanup()                        ; //Stop all accesses
 smq.cleanup()                        ;

 return(rc) ;                           //Return RC

}

// End of MQNQMTOQ.CPP source
